# -*- coding:Utf-8 -*-
import es, re, psyco, random, weaponlib
from dm_config import dmConfig
psyco.full()

importname = __import__(__name__)
regex = re.compile('//.*|(\r|\n|")+')
gamedir = es.getString("eventscripts_gamedir")
weaponorder = gamedir + "/cfg/dm_python/weapon/order.txt"
weaponfile = gamedir + "/cfg/dm_python/weapon/weapons.txt"

class DmWeapon(object):
    """ Weapon Management """

    def __init__(self):
        self.o_pistol = []
        self.o_shotgun = []
        self.o_smg = []
        self.o_rifle = []
        self.o_sniper = []
        self.o_grenade = []
        self.o_primary = []
        self.o_secondary = []

        self.pistol = []
        self.shotgun = []
        self.smg = []
        self.rifle = []
        self.sniper = []
        self.grenade = []
        self.primary = []
        self.secondary = []

        self.allowed = []
        self.weaponmenu = [
            "glock", "usp", "p228", "deagle", "fiveseven", "elite",
            "m3", "xm1014",
            "tmp", "mac10", "mp5navy", "ump45", "p90",
            "galil", "famas", "ak47", "m4a1", "sg552", "aug", "m249",
            "scout", "awp", "g3sg1", "sg550"]
        self.weaponlist = [
            "glock", "usp", "p228", "deagle", "fiveseven", "elite",
            "m3", "xm1014",
            "tmp", "mac10", "mp5navy", "ump45", "p90",
            "galil", "famas", "ak47", "m4a1", "sg552", "aug", "m249",
            "scout", "awp", "g3sg1", "sg550",
            "hegrenade", "flashbang", "smokegrenade"]

    def __getitem__(self, item):
        weaponInfo = weaponlib.getWeapon(item)
        if not weaponInfo: return None
        weaponTags = "#".join(weaponInfo.tags)
        if "#pistol" in weaponTags: return self.pistol
        elif "#shotgun" in weaponTags: return self.shotgun
        elif "#smg" in weaponTags: return self.smg
        elif "#sniper" in weaponTags: return self.sniper
        elif "#rifle" in weaponTags or "#machinegun" in weaponTags: return self.rifle
        elif "#grenade" in weaponTags: return self.grenade

    def __add__(self, item):
        weaponList = self[item]
        if isinstance(weaponList, list):
            if not item in self.allowed: self.allowed.append(item)
            if not item in weaponList: weaponList.append(item)
            if weaponList == self.pistol:
                if not item in self.secondary: self.secondary.append(item)
            elif weaponList <> self.grenade:
                if not item in self.primary: self.primary.append(item)

    def __sub__(self, item):
        weaponList = self[item]
        if isinstance(weaponList, list):
            if item in self.allowed: self.allowed.remove(item)
            if item in weaponList: weaponList.remove(item)
            if weaponList == self.pistol:
                if item in self.secondary: self.secondary.remove(item)
            elif weaponList <> self.grenade:
                if item in self.primary: self.primary.remove(item)

    def __contains__(self, item):
        return str(item).replace("weapon_", "") in self.allowed

    def i_read(self):
        try:
            openfile = open(weaponfile, "rb")
            lines = [line for line in map(lambda line: regex.sub("", line).strip(), openfile.readlines()) if line]
            openfile.close()
            return lines
        except IOError:
            es.dbgmsg(0, "dm_python: cannot read file cfg/dm_python/weapon/weapons.txt")
            return []

    def i_order(self):
        try:
            openfile = open(weaponorder, "rb")
            lines = []
            for line in map(lambda line: regex.sub("", line).strip(), openfile.readlines()):
                if line in self.weaponmenu and not line in lines: lines.append(line)
            openfile.close()
            return lines
        except IOError:
            es.dbgmsg(0, "dm_python: cannot read file cfg/dm_python/weapon/order.txt")
            return []

    def order(self):
        order = self.i_order()
        # check if all weapons are in the file
        if len(order) <> len(self.weaponmenu):
            openfile = open(weaponorder, "w")
            openfile.write('// order.txt for dm_python script\n// sort weapon into the deathmatch menu\n\n')
            for weapon in order: openfile.write("\n" + weapon)
            for weapon in self.weaponmenu:
                if not weapon in order:
                    order.append(weapon)
                    openfile.write("\n" + weapon)
            openfile.close()

        for weapon in order:
            weaponInfo = weaponlib.getWeapon(weapon)
            if weaponInfo:
                weaponTags = "#".join(weaponInfo.tags)
                if "#pistol" in weaponTags:
                    if not weapon in self.o_secondary: self.o_secondary.append(weapon)
                    if not weapon in self.o_pistol: self.o_pistol.append(weapon)

                else:
                    if not weapon in self.o_primary: self.o_primary.append(weapon)
                    if "#shotgun" in weaponTags:
                        if not weapon in self.o_shotgun: self.o_shotgun.append(weapon)
                    elif "#smg" in weaponTags:
                        if not weapon in self.o_smg: self.o_smg.append(weapon)
                    elif "#sniper" in weaponTags:
                        if not weapon in self.o_sniper: self.o_sniper.append(weapon)
                    elif "#rifle" in weaponTags or "#machinegun" in weaponTags:
                        if not weapon in self.o_rifle: self.o_rifle.append(weapon)

    def read(self):
        self.pistol, self.shotgun, self.smg, self.rifle, self.sniper, self.grenade, self.primary, self.secondary = [], [], [], [], [], [], [], []
        for weaponName in self.i_read(): self.__add__(weaponName)

    def save(self):
        try:
            openfile = open(weaponfile, "w")
            openfile.write('// weapon.txt for dm_python script\n// add "//" for restrict a weapon\n\n')
            for weapon in self.weaponlist: openfile.write("\n" + weapon if self.__contains__(weapon) else "\n// " + weapon)
            openfile.close()
        except IOError: es.dbgmsg(0, "dm_python: cannot read file cfg/dm_python/weapon/weapons.txt")

    def botPrimary(self):
        weaponBots = str(dmConfig.botPrimary)
        if weaponBots == "0": return None
        if weaponBots:
            weaponList = [weaponName for weaponName in map(lambda weaponName: weaponName.strip(), weaponBots.split(",")) if self.__contains__(weaponName)]
            if weaponList: return random.choice(weaponList)
        primary = self.shotgun + self.smg + self.rifle + self.sniper
        return random.choice(primary) if primary else None

    def botSecondary(self):
        weaponBots = str(dmConfig.botSecondary)
        if weaponBots == "0": return None
        elif weaponBots:
            weaponList = [weaponName for weaponName in map(lambda weaponName: weaponName.strip(), weaponBots.split(",")) if self.__contains__(weaponName)]
            if weaponList: return random.choice(weaponList)
        return random.choice(self.pistol) if self.pistol else None

# create DmWeapon object
dmWeapon = DmWeapon()
